@@ -57,12 +57,17 @@ void snd_compat_request_module(const char *name, ...);
 #define request_module(name, args...) snd_compat_request_module(name, ##args)
 #endif
 
-#if LINUX_VERSION_CODE < KERNEL_VERSION(2,6,0)
+#if LINUX_VERSION_CODE < KERNEL_VERSION(2,6,9)
 #include <linux/compiler.h>
+#ifndef __iomem
+#define __iomem
+#endif
 #ifndef __user
 #define __user
 #endif
+#endif /* < 2.6.9 */
 
+#if LINUX_VERSION_CODE < KERNEL_VERSION(2,6,0)
 /* for compat layer */
 #include <linux/pci.h>
 
@@ -177,6 +182,51 @@ static inline struct proc_dir_entry *PDE(const struct inode *inode)
 
 #endif /* < 2.6.0 */
 
+#include <linux/wait.h>
+#ifndef wait_event_timeout
+#if LINUX_VERSION_CODE >= KERNEL_VERSION(2, 6, 0)
+#define __wait_event_timeout(wq, condition, ret)			\
+do {									\
+	DEFINE_WAIT(__wait);						\
+									\
+	for (;;) {							\
+		prepare_to_wait(&wq, &__wait, TASK_UNINTERRUPTIBLE);	\
+		if (condition)						\
+			break;						\
+		ret = schedule_timeout(ret);				\
+		if (!ret)						\
+			break;						\
+	}								\
+	finish_wait(&wq, &__wait);					\
+} while (0)
+#else
+#define __wait_event_timeout(wq, condition, ret)			\
+do {									\
+	wait_queue_t __wait;						\
+	init_waitqueue_entry(&__wait, current);				\
+									\
+	add_wait_queue(&wq, &__wait);					\
+	for (;;) {							\
+		set_current_state(TASK_UNINTERRUPTIBLE);		\
+		if (condition)						\
+			break;						\
+		ret = schedule_timeout(ret);				\
+		if (!ret)						\
+			break;						\
+	}								\
+	current->state = TASK_RUNNING;					\
+	remove_wait_queue(&wq, &__wait);				\
+} while (0)
+#endif /* 2.6.0 */
+#define wait_event_timeout(wq, condition, timeout)			\
+({									\
+	long __ret = timeout;						\
+	if (!(condition))						\
+		__wait_event_timeout(wq, condition, __ret);		\
+	__ret;								\
+})
+#endif
+
 #ifndef CONFIG_HAVE_STRLCPY
 size_t snd_compat_strlcpy(char *dest, const char *src, size_t size);
 #define strlcpy(dest, src, size) snd_compat_strlcpy(dest, src, size)
@@ -197,6 +247,14 @@ int snd_compat_vsnprintf(char *buf, size_t size, const char *fmt, va_list args);
 #define vscnprintf(buf,size,fmt,args) vsnprintf(buf,size,fmt,args)
 #endif
 
+#ifndef CONFIG_HAVE_SSCANF
+#include <stdarg.h>
+int snd_compat_sscanf(const char * buf, const char * fmt, ...);
+int snd_compat_vsscanf(const char * buf, const char * fmt, va_list args);
+#define sscanf(buf,fmt,args...) snd_compat_sscanf(buf,fmt,##args)
+#define vsscanf(buf,fmt,args) snd_compat_vsscanf(buf,fmt,args)
+#endif
+
 #if defined(__alpha__) && LINUX_VERSION_CODE < KERNEL_VERSION(2, 3, 14)
 #include <asm/io.h>
 #undef writeb
@@ -364,6 +422,10 @@ enum {
 #define snd_usb_get_speed(dev) USB_SPEED_FULL
 #endif
 
+#if LINUX_VERSION_CODE < KERNEL_VERSION(2, 6, 8)
+#define usb_kill_urb(urb) usb_unlink_urb(urb)
+#endif
+
 #endif /* SND_NEED_USB_WRAPPER && CONFIG_USB */
 
 #if LINUX_VERSION_CODE >= KERNEL_VERSION(2, 5, 24) \
@@ -465,7 +527,14 @@ static inline void class_simple_device_add(struct class_simple *class, int devnu
 static inline void class_simple_device_remove(int devnum) { return; }
 #endif
 
-#include "amagic.h"
+/* msleep */
+#if LINUX_VERSION_CODE < KERNEL_VERSION(2, 6, 6)
+#define msleep(msecs) \
+	do { \
+		set_current_state(TASK_UNINTERRUPTIBLE); \
+		schedule_timeout(((msecs) * HZ + 999) / 1000);	\
+	} while (0)
+#endif
 
 #if LINUX_VERSION_CODE < KERNEL_VERSION(2, 6, 0)
 #define snd_card_set_dev(card,dev) /* no struct device */
@@ -482,14 +551,26 @@ static inline void class_simple_device_remove(int devnum) { return; }
 #include <linux/moduleparam.h>
 #undef module_param
 #define SNDRV_MODULE_TYPE_int	"i"
-#define SNDRV_MODULE_TYPE_bool	"b"
+#define SNDRV_MODULE_TYPE_bool	"i"
 #define SNDRV_MODULE_TYPE_uint	"i"
 #define SNDRV_MODULE_TYPE_charp	"s"
 #define SNDRV_MODULE_TYPE_long	"l"
-#define module_param_array(name, type, num, perm) \
+#define module_param_array(name, type, nump, perm) \
 	MODULE_PARM(name, "1-" __MODULE_STRING(SNDRV_CARDS) SNDRV_MODULE_TYPE_##type)
 #define module_param(name, type, perm) \
 	MODULE_PARM(name, SNDRV_MODULE_TYPE_##type)
+#elif LINUX_VERSION_CODE < KERNEL_VERSION(2, 6, 10)
+#include <linux/moduleparam.h>
+#undef module_param_array
+/* we assumme nump is always NULL so we can use a dummy variable */
+#define module_param_array(name, type, nump, perm) \
+	static int boot_devs_##name; \
+	module_param_array_named(name, name, type, boot_devs_##name, perm)
+#endif
+
+/* dump_stack hack */
+#ifndef CONFIG_HAVE_DUMP_STACK
+#define dump_stack()
 #endif
 
 /* power management compatibility layer */
@@ -502,5 +583,13 @@ void snd_card_pci_resume(struct pci_dev *dev);
 #endif
 #endif
 
+/* kcalloc */
+#ifndef CONFIG_HAVE_KCALLOC
+#ifndef CONFIG_SND_DEBUG_MEMORY
+void *snd_compat_kcalloc(size_t n, size_t size, int gfp_flags);
+#define kcalloc(n,s,f) snd_compat_kcalloc(n,s,f)
+#endif
+#endif
+
 
 #endif /* __SOUND_LOCAL_DRIVER_H */
