@@ -1,21 +1,4 @@
 /*******************************************************************************
- *  azx-alsa-drop-driver-1.1.2.tgz is a driver for hardware using Azaila arch.
- *  Copyright (C) 2004 Intel Corporation
- *
- *  This program is free software; you can redistribute it and/or
- *  modify it under the terms of the GNU General Public License
- *  as published by the Free Software Foundation; either version 2
- *  of the License, or (at your option) any later version.
- *
- *  This program is distributed in the hope that it will be useful,
- *  but WITHOUT ANY WARRANTY; without even the implied warranty of
- *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- *  GNU General Public License for more details.
- *
- *  You should have received a copy of the GNU General Public License
- *  along with this program; if not, write to the Free Software
- *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
- *
  *
  *  azx_controller.h - HD Audio hardware Interface.
  *
@@ -35,9 +18,6 @@
  *  this program; if not, write to the Free Software Foundation, Inc., 59
  *  Temple Place - Suite 330, Boston, MA  02111-1307, USA.
  *
- *  The full GNU General Public License is included in this distribution in the
- *  file called LICENSE.
- *
  *  CONTACTS:
  *
  *  Matt Jared		matt.jared@intel.com
@@ -45,51 +25,17 @@
  *  Dan Kogan		dan.d.kogan@intel.com
  *
  *
- *  azx_controller.h,v 1.1 2004/07/28 14:55:35 tiwai Exp
+ *  azx_controller.h,v 1.3 2004/09/22 15:26:31 tiwai Exp
  *
  *  azx_controller.h,v
  *  tiwai
- *  2004/07/28 14:55:35
+ *  2004/09/22 15:26:31
  *
  ******************************************************************************/
 
-
 #ifndef ___AZX_CONTROLLER___
 #define	___AZX_CONTROLLER___
 
-
-#include <sound/driver.h>
-#include <asm/io.h>
-#include <linux/delay.h>
-#include <linux/interrupt.h>
-#include <linux/init.h>
-#include <linux/pci.h>
-#include <linux/slab.h>
-#include <linux/gameport.h>
-#include <sound/core.h>
-#include <sound/pcm.h>
-#include <sound/info.h>
-#include <sound/mpu401.h>
-#include <sound/initval.h>
-
-#define SNDRV_GET_ID
-
-// *********************************************************************
-// control which codec is used by commenting/uncommenting
-// the following directive.  If commented out, Relatek codec
-// will be used
-
-#define ___DO_C_MEDIA___
-// *********************************************************************
-
-// *********************************************************************
-// control if using internel sample audio or external PCM file
-
-//#define __DO_AZX_SINE_SAMPLE__
-// *********************************************************************
-
-
-
 /*
  * AZALIA Interface Controller Registers
  *  Symbol		       		OFFSET	MEANING			     Bytes 1st Last
@@ -710,139 +656,6 @@
 
 /* ---------------------------- end REGISTER MASK and MACROs ------------------------------- */
 
-typedef struct{
-	unsigned int channels;
-	unsigned int rate;
-	unsigned int format;
-	dma_addr_t dma_area;
-	snd_pcm_uframes_t buffer_size;
-	snd_pcm_uframes_t period_size;
-}pcm_hw_update_t;
-
-
-
-/**
- * struct azx_controller - Azalia controller public declaration.
- *
- * Internal structure declaration and private function definitions
- * in azx_main.c
- *
- **/
-struct azx_controller;
-
-u8	azx_dev_readb  (struct azx_controller* azx, unsigned int reg_offset);
-void	azx_dev_writeb (struct azx_controller* azx, unsigned int reg_offset, u8 new_val);
-u16	azx_dev_readw  (struct azx_controller* azx, unsigned int reg_offset);
-void	azx_dev_writew (struct azx_controller* azx, unsigned int reg_offset, u16 new_val);
-u32	azx_dev_readl  (struct azx_controller* azx, unsigned int reg_offset);
-void	azx_dev_writel (struct azx_controller* azx, unsigned int reg_offset, u32 new_val);
-
-
-int  __devinit azx_dev_start     (struct pci_dev *pci, const struct pci_device_id *pci_id);
-void __devexit azx_dev_end       (struct pci_dev *pci);
-
-
-
-
-/*
- * TODO:
- *    - define get/set (word, long, byte) functions (framework)
- *    - kill literals (named masks)
- *    - track down all places where registers are written and make sure that
- *      they read/written according to their documented size
- *    - regularize azx_dev fns to take a azx_controller*
- *    - fix all readw to use azx_dev_readw
- */
-
-
-enum{INPUT_STREAM_1, INPUT_STREAM_2, INPUT_STREAM_3, INPUT_STREAM_4, OUTPUT_STREAM_1, OUTPUT_STREAM_2, OUTPUT_STREAM_3, OUTPUT_STREAM_4, CLEAR_SIE};
-
-// controller API
-int 	azx_dev_init_pci            			( struct pci_dev *pci, const struct pci_device_id *pci_id );
-void    azx_dev_test_ringbuffers    			( struct azx_controller* azx );
-int     azx_dev_count_codecs        			( struct azx_controller* azx );
-int     azx_dev_reset_controller    			( struct azx_controller* azx );
-int     azx_dev_count_codecs        			( struct azx_controller* azx );
-int		azx_dev_set_int_enable	    			( struct azx_controller* azx, int is_turnon );
-
-int azx_dev_set_stream_int_enable( struct azx_controller* azx, u32 sie_mask, int is_turnon );
-int		azx_dev_set_unsol_response  			( struct azx_controller* azx, int is_turnon );
-int		azx_dev_set_controller	    			( struct azx_controller* azx, int is_turnon );
-int		azx_dev_set_unsol_response  			( struct azx_controller* azx, int is_turnon );
-int		azx_dev_set_int_enable	    			( struct azx_controller* azx, int is_turnon );
-int		azx_dev_set_globl_int_enable			( struct azx_controller* azx, int is_turnon );
-void	azx_dev_reset_sdiwake 	    			( struct azx_controller* azx );
-void 	azx_dev_reset_sdiwen	    			( struct azx_controller* azx );
-void    azx_dev_reset_statests      			( struct azx_controller* azx );
-void	azx_dev_show_state	    				( struct azx_controller* azx );
-void	azx_dev_show_regs	    				( struct azx_controller* azx );
-void	azx_dev_show_pci_cfg	    			( struct pci_dev *pdev );
-int		azx_dev_selftest	    				( struct azx_controller* azx );
-u8* 	azx_dev_return_base_addr				(struct azx_controller* azx);
-struct 	azx_ring_buffer* azx_dev_return_corb	(struct azx_controller* azx);
-struct 	azx_ring_buffer* azx_dev_return_rirb	(struct azx_controller* azx);
-int 	azx_dev_reset_controller_old			(u8 *azbar);
-void     azx_ctlr_inc_stream_int_counter(void);
-void     azx_ctlr_dec_stream_int_counter(void);
-int     azx_ctlr_get_stream_int_counter(void);
-
-
-
-
-
-// added by matt
-struct azx_controller* azx_dev_get_controller_inst(void);
-unsigned int set_azx_controller_irq(struct azx_controller* azx, int irq);
-int get_azx_controller_irq(struct azx_controller* azx);
-unsigned int set_stream_desc_control(struct azx_controller* azx);
-int get_azx_controller_id(void);
-int is_controller_active(void);
-struct pci_dev * get_azx_controller_pci(void);
-
-
-	
-// codec configuration functions
-int	 	azx_dev_cmedia_codec_setup				(struct azx_controller* azx, pcm_hw_update_t pcm_info);	
-int 	azx_dev_relatek_codec_setup				(struct azx_controller* azx, pcm_hw_update_t pcm_info);
-
-// stream functions
-int 	azx_dev_start_stream_run_bit			(struct azx_controller* azx);
-int 	azx_dev_stop_stream_run_bit				(struct azx_controller* azx);	
-
-// code for doing sample test (sine wave) ******************************
-int 	azx_dev_audio_test						(struct pci_dev *pci, struct azx_controller* azx);
-int 	azx_dev_audio_test_clean_up				(struct pci_dev* pci);
-// *********************************************************************
-
-
-
-// BDL Object
-
-#define MAX_BDL_BUF_SIZE (1024*64)
-
-unsigned int     azx_bdl_setup(struct pci_dev* pci, struct azx_controller* azx, pcm_hw_update_t pcm_info);
-unsigned int     azx_bdl_cleanup(struct pci_dev* pci);
-unsigned int     get_bdl_buff_a_addrs(dma_addr_t* phy_addr, int** virt_addr);
-unsigned int     get_bdl_buff_b_addrs(dma_addr_t* phy_addr, int** virt_addr);
-unsigned int     get_bdl_IOC_state(void);
-unsigned int     set_bdl_IOC_state(int state);
-unsigned int     clear_bdl_IOC_state(void);
-unsigned int     set_bdl_virt_address(int * addr);
-unsigned int     set_bdl_phy_address(dma_addr_t addr);
-unsigned int     add_sample_to_bdl_buffs(void);
-unsigned int     add_pcm_to_bdl_buffs(int buff_location, snd_pcm_runtime_t *runtime);
-struct azx_bdl*  get_bdl(void);
-u32* get_bdl_virtual(void);
-u32* get_bdl_head(void);
-void set_bdl_head(u32* bdl_ptr);
-
-// DMA monitoring
-u32    get_dma_buffer_position(void);
-
-
-u32 get_dma_bytes(void);
 
 #endif //___AZX_CONTROLLER___
 
-
-
